/**************************************************************************
 *									  *
 * 		 Copyright (C) 1989-1995 Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/

#ifndef __SYS_ARSESS_H__
#define __SYS_ARSESS_H__

#ident "$Revision: 1.5 $"

/*
 * Array Sessions
 *
 * An array session is a group of processes all related to each other by a
 * single unique identifier, the "array session handle". The processes
 * don't necessarily have to belong to the same parent-child chain, and
 * don't even have to be running on the same system. However, the default
 * is for a child process to inherit the array session handle of its
 * parent, so in the average case the processes in an array session are
 * parents/siblings/children of each other and reside on the same system.
 *
 * The goal of an array session is to correlate all the processes that
 * belong conceptually to the same login session or batch job, even if
 * those processes are running on several separate machines in an array.
 * Then, with the help of external software, the array session can
 * potentially be treated as a single unit for the purposes of accounting,
 * checkpoint/restart, job control, etc.
 *
 * A process can create a new array session and place itself in it by using
 * the "newarraysess" call. Ordinarily, this would only be done programs
 * like login or rshd, which are effectively logging a user into the
 * system, and programs like batch queue systems or crond, which are
 * running jobs on behalf of another user. A new array session is assigned
 * an array session handle that is unique to the current system only. If
 * the process is actually part of an existing array session on another
 * system (e.g. the process was spawned by mpirun or whatever) it can
 * change the handle of its array session to that of its "parent" on the
 * remote system using the "setash" call. The parent's array session handle
 * would presumably be unique across the entire array, although it would be
 * the parent's responsibility to arrange for that. The range of values
 * that the kernel will assign as array session handles is configurable so
 * that it would be possible to design a server to provide array-unique
 * handles; that is left as an exercise to the reader.
 *
 * An array session is *not* the same thing as a POSIX session, though the
 * two concepts are similar (not identical) on a single system. The most
 * important characteristic of a POSIX session is that all of the processes
 * included within it share a single controlling terminal. That distinction
 * does not exist for array sessions: processes from two different array
 * sessions might both be using the same controlling terminal (e.g. by way
 * of the newproj command) or contrarily two processes in the same array
 * session might have two different controlling terminals (e.g. when
 * running on different systems). However, in the simple case of a local
 * login with no remote interactions an array session and a POSIX session
 * would include the same set of processes.
 */

#include <sys/types.h>
#include <sys/extacct.h>
#include <sys/proc.h>
#include <sys/sema.h>


#define ASH_NONE	0	/* "No array session handle assigned" */
#define PRID_NONE	0	/* "No project ID assigned" */


#if defined(_KERNEL) || defined(_KMEMUSER)

typedef struct arsess {
	ash_t		as_handle;	/* array session handle */
	prid_t		as_prid;	/* project ID */

	lock_t		as_lock;	/* update lock */
	struct arsess	*as_next;	/* next arsess in act/free list */
	struct arsess	*as_prev;	/* previous arsess in act list */

	int		as_refcnt;	/* reference count */
	time_t		as_start;	/* start time (secs since 1970) */
	time_t		as_ticks;	/* lbolt at start */
	pid_t		as_pid;		/* pid that started this session */
	ushort_t	as_flag;	/* various flags */
	char		as_nice;	/* initial nice value of as_pid */

	/* Accounting data */
	acct_spi_t	as_spi;		/* Service Provider Information */
	acct_timers_t	as_timers;	/* accounting timers */
	acct_counts_t	as_counts;	/* accounting counters */
} arsess_t;

/* as_flag */
#define ASVALID	0x0001		/* array session is currently valid/in use */

#endif  /* _KERNEL || _KMEMUSER */


#ifdef _KERNEL

/* externally defined data */
extern arsess_t	arsess0;	/* pseudo-arsess for proc 0 */
extern arsess_t	*arsess;	/* the arsess table itself */
extern arsess_t	*asactlist;	/* array session active list */
extern ash_t	dfltash;	/* default array session handle */
extern prid_t	dfltprid;	/* default project ID */
extern int	do_sessacct;	/* non-zero => array session accting enabled */
extern int	use_astbl;	/* non-zero => use arsess table */
extern int	asmachid;	/* global ASH machine ID */


/* Array session functions */
extern void	arsess_accum(proc_t *, acct_timers_t *, acct_counts_t *);
extern arsess_t *arsess_alloc(void);
extern void	arsess_attach(proc_t *, arsess_t *);
extern int	arsess_enumashs(ash_t *, int);
extern arsess_t *arsess_find(ash_t);
extern void	arsess_free(arsess_t *);
extern int	arsess_getmachid(void);
extern int	arsess_join(proc_t *, ash_t);
extern void	arsess_leave(proc_t *);
extern int	arsess_new(proc_t *);
extern int	arsess_setash(ash_t, ash_t);
extern int	arsess_setmachid(int);
extern int	arsess_setprid(ash_t, prid_t);
extern int	arsess_setspinfo(ash_t, acct_spi_t *);

#endif	 /* _KERNEL */
#endif   /* _SYS_ARSESS_H */
