/**************************************************************************
 *									  *
 * 		 Copyright (C) 1995, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/
#ifndef __SYS_GFX_FLOW_H__
#define __SYS_GFX_FLOW_H__

#ident "$Revision: 1.1 $"

/*
 * gfx_flow.h -- platform-independent interfaces to graphics flow control
 *
 * Some platforms have built-in support for graphics flow control.
 * These interfaces hide platform dependencies while not exposing
 * I/O-bus or graphics-device dependencies.  The general rule is
 * that the graphics driver may contain bus/device dependencies but
 * should go through interfaces like these for platform dependencies.
 * Conversely, these interfaces should not contain any graphics-device
 * dependencies.  The implementations of these interfaces can be found in 
 * gfx/kern/graphics/<platform>.
 */

#include <sys/types.h>
#include <sys/edt.h>
#include <sys/iobus.h>

/*
 * gfx_flow_alloc() allocates resources that a given graphics 
 * device will use for its flow control.  It returns an opaque 
 * gfx_flow_hdl which is used for further operations.  NULL is returned 
 * if no more resources are available.  The caller also specifies the 
 * number of nanoseconds the used credits may be above a platform-
 * independent critical hi-water mark before the CPU will be interrupted and
 * the delayed_hiwater_intr_func will be called with the 
 * delayed_hiwater_intr_func_arg.  
 *
 * gfx_flow_dealloc() deallocates the per-device resources, which 
 * must no longer be in use.
 *
 * gfx_flow_wg_info_get() is called once to obtain information
 * about the WG.  is_present and is_built_in indicate
 * if the WG exists and, if so, whether it lives inside or outside
 * the CPU.  Endianess is set to little (0) or big (1).  Cache_algo is
 * set to one of cache algorithms defined in sys/immu.h.
 * Input_ptr is set to the virtual kernel address to which 32-bit words
 * can be written via the WG.  Again, these values are 
 * the same for all CPUs on the platform.
 */
typedef struct gfx_flow_info *	gfx_flow_hdl_t;
typedef void (*gfx_flow_intr_func_t)(void * arg, int is_user_mode);
typedef struct {
	uint_t		is_present;
	uint_t		is_built_in;
	uint_t		endianess;
	uint_t		cache_algo;
	volatile int *	input_ptr;
} gfx_flow_wg_info_t;

extern gfx_flow_hdl_t gfx_flow_alloc( 
	      	iopaddr_t 	   	gfifo_paddr,
		uint_t		   	total_credit_bytes,
		uint_t    	   	delayed_hiwater_intr_counter_nsec,
		gfx_flow_intr_func_t 	delayed_hiwater_intr_func,
	      	void *		   	delayed_hiwater_intr_func_arg );
extern void	      gfx_flow_dealloc( gfx_flow_hdl_t gfx_flow_hdl );
extern void 	      gfx_flow_wg_info_get( gfx_flow_hdl_t      gfx_flow_hdl,
					    gfx_flow_wg_info_t *info_ptr );

/*
 * The following calls can be made only when rescheduling is disabled on
 * the current CPU.  They are typically called in the CPU context-switch path.
 * 
 * gfx_flow_attach_me() attaches the current CPU to the target graphics
 * device.  The platform is programmed with credit_bytes and with the 
 * interrupt information passed to gfx_flow_alloc().   The caller is 
 * responsible for programming credit registers on the graphics-device side 
 * and for ensuring that the values on both sides add up to a consistent 
 * value.  This call should be made BEFORE enabling credit-flow-control on 
 * the graphics-device side.
 *
 * gfx_flow_detach_me() detaches the current CPU from the target graphics 
 * device.  It returns the total number of credits that the platform
 * has left.  This call should be made AFTER disabling credit-flow-control
 * on the graphics-device side.
 *
 * gfx_flow_flush_my_wg() flushes the calling CPU's write gatherer,
 * but does not wait for the flushed data to reach the graphics device.
 * If the caller desires this functionality, it must typically load
 * a register on the graphics device.  The <wait> parameter in this
 * interface only waits for the data to be flushed out of the WG.
 */
extern void 	gfx_flow_attach_me( gfx_flow_hdl_t gfx_flow_hdl );
extern void	gfx_flow_detach_me( gfx_flow_hdl_t gfx_flow_hdl );
extern void	gfx_flow_flush_my_wg( gfx_flow_hdl_t gfx_flow_hdl,
				      uint_t	     wait );

#endif /* __SYS_GFX_FLOW_H__ */
