/*                                                                        *
 *               Copyright (C) 1989, Silicon Graphics, Inc.               *
 *                                                                        *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *                                                                        *
 **************************************************************************/

#ident "$Revision: 1.21 $"
#ifndef _GRIO_H_
#define _GRIO_H_

/*
 * This include file contains the definitions and structures 
 * used by the GRIO subsystem
 */

/*
 * Global defines that determine the amount of resources used 
 * by the GRIO subsystem.
 */

/*
 * maximum number of disks used by the GRIO subsystem
 */
#define	MAX_NUM_DISKS		512	

/*
 * maximum number of disks in a single stripped volume used by GRIO
 */
#define	MAX_ROTOR_SLOTS		256

/*
 * maximum number of reservations allowed in GRIO subsystem
 */
#define MAX_NUM_RESERVATIONS	3000

/*
 * maximum depth of ggd/grio command queue
 */
#define MAX_GRIO_QUEUE_COUNT    200

/* 
 * maximum number of streams returned by a single STAT call
 */
#define	MAX_STREAM_STAT_COUNT	MAX_NUM_RESERVATIONS

/* 
 * maximum size of a GRIO device name (as recorded in the /etc/grio_config file)
 */
#define DEV_NMLEN		32

/*
 * maximum number of GRIO streams allowed without a license
 */
#define DEFAULT_GRIO_STREAMS    4

/* 
 * maximum number of GRIO streams allowed with an "unlimited" license
 */
#define UNLIMITED_GRIO_STREAMS  MAX_NUM_RESERVATIONS

/* 
 * redefine type definitions
 */
typedef	__uint64_t	gr_ino_t;
typedef	uuid_t		stream_id_t;

/* 
 * grio_resv structure is filled in by the user process and is sent to the
 * library routine grio_request() along with the file descriptor of the 
 * resouce for which the I/O rate guarantee is being requested.
 * 
 * The grio_request() call will return 0 if there are no errors. If there
 * is an error the routine will return -1 and the gr_error field will 
 * contain the error number. In addition, if the guarantee request is
 * denied due to lack of device bandwidth, then gr_optime, and gr_opsize
 * will contain values describing the maximum remaining bandwidth.
 *
 *
 * Return errors are:
 * 	The first two errors indicate an error in the library.
 *		EINVAL		- could not communicate to daemon
 *		ESRCH		- invalid procid 
 *
 *	These errors indicate an error in the calling process.
 *		EBADF		- could not stat file or file already
 *				  has a guarantee
 *		EIO		- error in guarantee request structure 
 *				   * start or duration time is incorrect
 *				   * invalid flags in gr_flags field
 *		EPERM		- invalid I/O size for file system
 *		ENOSPC		- bandwidth could not be allocated
 *		ENOENT		- file does not contain any extents.
 *		EACCES		- cannot provide desired level of 
 *				  guarantee (i.e HARD vs SOFT)
 */
typedef struct grio_resv {
	time_t		gr_start;		/* when to start in secs      */
	time_t		gr_duration;		/* len of guarantee in secs   */
	time_t		gr_optime;		/* time of one op in usecs    */
	int		gr_opsize;		/* size of each op in bytes   */
	stream_id_t	gr_stream_id;		/* stream id                  */
	int		gr_flags;		/* flags field                */
	int		gr_error;		/* returned: error code       */
	char		gr_errordev[DEV_NMLEN];	/* device that caused error   */
} grio_resv_t;

typedef struct irix5_grio_resv {
	time_t		gr_start;		/* when to start in secs      */
	time_t		gr_duration;		/* len of guarantee in secs   */
	time_t		gr_optime;		/* time of one op in usecs    */
	int		gr_opsize;		/* size of each op in bytes   */
	stream_id_t	gr_stream_id;		/* stream id		      */
	int		gr_flags;		/* flags field                */
	int		gr_error;		/* returned: error code       */
	char		gr_errordev[DEV_NMLEN];	/* device that caused error   */
} irix5_grio_resv_t;

/* 
 * This structure is used to return statistics info to the caller.
 * It is used with GRIO_GET_INFO resv_type.
 *	subcommands are:
 *		GRIO_DEV_RESVS:
 *			return number of reservations on the device 
 *			identified by ( device_name) in grio_blk_t
 *			structure.
 *		GRIO_FILE_RESVS:	
 *			return number of reservations on the file 
 *			identified by ( fs_dev, ino) pair in grio_blk_t
 *			structure.
 *		GRIO_PROC_RESVS:
 *			return number of reservations for the process 
 *			identified by ( procid ) in grio_blk_t structure.
 */
typedef struct grio_stats {
	/*
	 * value dependent on the subcommand and grio_blk_t parameters.
	 */
	u_long	gs_count;		/* current number of reservations
					 * active on the device/file/proc
					 */
	u_long	gs_maxresv;		/* maximum number of reservations
					 * allowed on the device.
					 * for file/proc this is the number
					 * of licensed streams.
					 */
	u_long	gs_optiosize;		/* size of the optimal i/o size
					 * in bytes for this device.
					 * not defined for file/proc.
					 */
	char 	devname[DEV_NMLEN];
} grio_stats_t;

typedef struct irix5_grio_stats {
	__uint32_t	gs_count;
	__uint32_t	gs_maxresv;
	__uint32_t	gs_optiosize;
	__uint32_t	gs_resvcount;
	__uint32_t	gs_unresvcount;
	__uint32_t	gs_resvdeniedcount;
} irix5_grio_stats_t;

/* 
 * Defines for the gr_flags field.
 * Set the user process or by the grio_lib
 */
#define EXTENT_ALIGNED		0x00000001
#define HARD_GUAR		0x00000002
#define SOFT_GUAR		0x00000004

#define	PER_FILE_GUAR		0x00000008
#define	PER_FILE_SYS_GUAR	0x00000010

#define PROC_PRIVATE_GUAR	0x00000020	
#define PROC_SHARE_GUAR		0x00000040

#define FIXED_ROTOR_GUAR	0x00000100
#define SLIP_ROTOR_GUAR		0x00000200
#define NON_ROTOR_GUAR		0x00000400

#define DEADLINE_SCHED_GUAR	0x00001000
#define REALTIME_SCHED_GUAR	0x00002000
#define NON_SCHED_GUAR		0x00004000

#define SYSTEM_GUAR		0x00008000

#define GUARANTEE_MASK		0x0000FFFF

/*
 * Defines for the types of reservations
 * Set by the ggd daemon
 */
#define RESERVATION_STARTED	0x00010000
#define RESERVATION_TYPE_VOD	0x00020000
#define RESERVATION_TYPE_FS	0x00040000
#define RESERVATION_IGNORE	0x00080000

#define RESERVATION_MASK	0x00FF0000

/*
 * Defines for stream states. 
 * Set by the grio driver.
 */
#define	STREAM_REMOVE_IN_PROGRESS	0x01000000
#define	STREAM_INITIATE_IN_PROGRESS	0x02000000

#define STREAM_SLIPPED_ONCE		0x04000000

#define STREAM_STATE_MASK		0xFF000000

/*
 * backwards compatability
 */
#define HARD_GUARANTEE		HARD_GUAR
#define SOFT_GUARANTEE		SOFT_GUAR
#define VOD_LAYOUT		SLIP_ROTOR_GUAR

#define IS_VOD_GUAR( griorp ) \
	( (griorp->gr_flags & FIXED_ROTOR_GUAR)  || \
	  (griorp->gr_flags & SLIP_ROTOR_GUAR)  )

#define IS_FILESYS_GUAR( griorp ) \
	( griorp->gr_flags & PER_FILE_SYS_GUAR )

#define IS_FILE_GUAR( griorp ) \
	( griorp->gr_flags & PER_FILE_GUAR )

/*
 * external declarations for lib_grio routines
 */
extern int grio_unreserve_bw( stream_id_t *);
extern int grio_reserve_file( int, grio_resv_t *);
extern int grio_reserve_fs( dev_t, grio_resv_t *);
extern int grio_query_fs( dev_t, int, int *, int *);
extern int grio_query_fs_bw( dev_t, int, int);
extern int grio_associate_file( int, stream_id_t *);
extern int grio_pass_bandwidth( int, char *, int );

/*
 * routines for backward compatibility only
 */
extern int grio_request(int, grio_resv_t *);
extern int grio_remove_request(int, grio_resv_t *);

/*
 * The grio_request() library routine fills in this structure and writes
 * it to the ggd deamon.
 */
typedef struct grio_command {
        int             gr_cmd;			/* grio command            */
	int		gr_subcmd;		/* grio subcommand	   */
        dev_t           gr_fs_dev;		/* file system dev_t       */
        pid_t           gr_procid;		/* process id of requestor */
        gr_ino_t       	gr_ino;			/* inode number            */
	union {
        	grio_resv_t     gr_resv;      	/* grio request info       */
		grio_stats_t	gr_stats;	/* grio stats info	   */
	} cmd_info;
	int		gr_bytes_bw;		/* returned bandwidth	   */
	int		gr_usecs_bw;		/* returned bandwidth	   */
	int		gr_time_to_wait;	/* kernel wait time	   */
} grio_cmd_t;


typedef struct irix5_grio_cmd {
        int             gr_cmd;
	int		gr_subcmd;
        dev_t           gr_fs_dev;
        pid_t           gr_procid;
        gr_ino_t	gr_ino;
	union {
        	irix5_grio_resv_t       gr_resv;
		irix5_grio_stats_t	gr_stats;
	} cmd_info;
	int		gr_bytes_bw;
	int		gr_usecs_bw;
	int		gr_time_to_wait;
} irix5_grio_cmd_t;

/*
 * Structure definitions with fields defined as known sizes.
 * This is done to have a layer of abstraction between the kernel and the
 * ggd daemon. The kernel can have 32 bit inumber, file sizes, and file
 * system sizes, but the ggd will always view them as 64 bit quantities.
 */
typedef struct grio_file_id {
	gr_ino_t		ino;
	dev_t		fs_dev;
	pid_t		procid;
} grio_file_id_t;

typedef struct grio_disk_id {
	int		num_iops;
	int		opt_io_size;
	time_t		iops_time;
} grio_disk_id_t;

typedef struct grio_bmbt_irec {
	__uint64_t	br_startoff;
	__uint64_t	br_startblock;
	__uint64_t	br_blockcount;
} grio_bmbt_irec_t;


#define GRIO_PIPE               "/tmp/grio.pipe"
#define GRIO_PIPE_STRLEN        14
#define GRIO_PID_LEN            10
#define GRIO_PIPE_NAMLEN        GRIO_PIPE_STRLEN + GRIO_PID_LEN
#define GRIO_LOCK_FILE		"/tmp/grio.lock"

/*
 * defines for the gr_cmd field.
 */
#define GRIO_UNKNOWN		0
#define GRIO_RESV		1
#define	GRIO_UNRESV		2
#define GRIO_UNRESV_ASYNC	3
#define GRIO_PURGE_VDEV_ASYNC	4
#define	GRIO_GET_STATS		5
#define	GRIO_GET_BW		6
#define GRIO_ACTIVE_COMMANDS	GRIO_GET_BW + 1
#define GRIO_MAX_COMMANDS	20

/*
 * this structure defines statisical 
 * information maintained by the ggd daemon
 */
typedef struct ggd_node_summary {
	char	node_name[DEV_NMLEN];
	int	ctlr_n_dev;			/* disk location      */
	int	own_index;
	int	child_index;
	int	sibling_index;
	int	max_io_rate;			/* rate in kbytes/sec */
	int	remaining_io_rate;		/* rate in kbytes/sec */
	int	optiosize;			/* opt i/o size in bytes */
	int	num_opt_ios;			/* max number of opt i/os */
} ggd_node_summary_t;

typedef struct ggd_info_summary {
	int	rev_count;			/* updated when ggd restarts */
	int	ggd_info_magic;			/* magic number */
	int	num_nodes;			/* num enteries in node_info */
	int	active_cmd_count;		/* num enteries in per_cmd_  */
	int	max_cmd_count;			/* set to GRIO_MAX_COMMANDS  */
	int	per_cmd_count[GRIO_MAX_COMMANDS];
	ggd_node_summary_t node_info[1];
} ggd_info_summary_t;

#define	GGD_INFO_KEY		0x53637445
#define GGD_INFO_MAGIC		0x53637445

#define	CUL_ENCODE( ctlr, unit, lun )  ( 	(ctlr << 16) | \
						(unit << 8 ) | \
						( lun ) )

#define	CUL_DECODE( code, ctlr, unit, lun ) {			\
			ctlr = (code >> 16 ) & 0xfff;		\
			unit = (code >> 8 )  & 0xff;		\
			lun  = (code ) & 0xff;			\
		}


/*
 * Subcommands for the GRIO_GET_STATS command.
 */
#define GRIO_FILE_RESVS		1
#define GRIO_DEV_RESVS		2
#define GRIO_PROC_RESVS		3

#define GRIO_ASYNC_CMD(griocp)	\
	(( (griocp)->gr_cmd == GRIO_UNRESV_ASYNC ) || \
	 ( (griocp)->gr_cmd == GRIO_PURGE_VDEV_ASYNC) )

#define GRIO_STATS_REQ(grioreq)	\
	( (grioreq)->gr_cmd == GRIO_GET_STATS )

#define	GRIO_GET_RESV_DATA( griocp )		(&(griocp->cmd_info.gr_resv))
#define	GRIO_GET_STATS_DATA( griocp )		(&(griocp->cmd_info.gr_stats))

#define GET_VOL_STRIPE_SIZE( volep ) \
		(volep->grp_size * volep->stripe_unit_size * NBPSCTR)

#define ROUND_UP( amount, alignment ) \
		(((amount + alignment - 1)/alignment) *alignment)


/*
 * syssgi SGI_GRIO commands
 */
#define GRIO_ADD_DISK_INFO   		21
#define GRIO_UPDATE_DISK_INFO   	22
#define GRIO_ADD_STREAM_INFO		23
#define	GRIO_ADD_STREAM_DISK_INFO	24
#define GRIO_ASSOCIATE_FILE_WITH_STREAM	25
#define	GRIO_REMOVE_STREAM_INFO		26
#define	GRIO_REMOVE_ALL_STREAMS_INFO	27
#define GRIO_GET_FILE_EXTENTS   	28
#define GRIO_GET_FS_BLOCK_SIZE  	29
#define GRIO_GET_FILE_RT		30
#define GRIO_WRITE_GRIO_REQ 		31
#define GRIO_READ_GRIO_REQ 		32
#define GRIO_WRITE_GRIO_RESP		33
#define GRIO_GET_STREAM_ID		34
#define GRIO_GET_ALL_STREAMS		35
#define GRIO_GET_VOD_DISK_INFO		36

#define GRIO_MONITOR_START		37
#define GRIO_MONITOR_GET		38
#define GRIO_MONITOR_END		39

#define COPY_STREAM_ID(from, to)     ( bcopy(&(from),&(to), sizeof(uuid_t)) )
#define EQUAL_STREAM_ID(one, two)    ( uuid_equal( &one, &two, &status) )

typedef struct grio_add_disk_info_struct {
	dev_t	gdev;
	int	num_iops_rsv;
	int	num_iops_max;
	int	opt_io_size;
	int	rotation_slot;
} grio_add_disk_info_struct_t;


typedef struct grio_stream_stat {
	stream_id_t		stream_id;
	gr_ino_t		ino;
	dev_t			fs_dev;
	pid_t			procid;
} grio_stream_stat_t;

typedef struct grio_vod_info {
	int			num_rotor_slots;
	int			rotor_position;
	int			num_rotor_streams;
	int			num_nonrotor_streams;
	int			rotor_slot[MAX_ROTOR_SLOTS];
} grio_vod_info_t;


#ifdef _KERNEL
/*
 * Kernel buffer scheduling structure.
 */
/*
 * grio disk information
 * the kernel allocates and maintains one such structure for 
 * each disk used by the grio subsystem
 */
typedef struct grio_disk_info {
        int             		num_iops_max;
        int             		num_iops_rsv;
        int             		opt_io_size;
	int				active;
	lock_t          		lock;
	struct grio_stream_disk_info	*diskstreams;

	int				ops_issued;
	int				ops_complete;
	int				subops_issued;
	int				subops_complete;
	int				rotate_position;
	
	time_t				time_start;
	int				opcount;
	time_t				reset_time;
	time_t				timeout_time;
	toid_t				timeout_id;
	dev_t				diskdev;
} grio_disk_info_t;

#define GRIO_STREAM_TABLE_SIZE	50
#define GRIO_STREAM_TABLE_INDEX( id )	 ( id % GRIO_STREAM_TABLE_SIZE )

/*
 * grio stream information 
 * the kernel allocates and maintains one such structure for
 * each active grio stream
 */
typedef struct grio_stream_info {
	struct grio_stream_info 	*nextstream;
	struct grio_stream_disk_info	*diskstreams;
	lock_t				lock;
	stream_id_t			stream_id;
	pid_t				procid;
	gr_ino_t			ino;
	dev_t				fs_dev;
	int				flags;
	int				total_slots;
	int				max_count_per_slot;
	int				rotate_slot;
	time_t				last_stream_op;
} grio_stream_info_t;



/*
 * Macros to access the per stream flags.
 */
#define MARK_STREAM_AS_BEING_REMOVED(griosp) \
			(griosp->flags |= STREAM_REMOVE_IN_PROGRESS)

#define MARK_STREAM_AS_INITIATING_IO(griosp) \
			(griosp->flags |= STREAM_INITIATE_IN_PROGRESS)

#define CLEAR_STREAM_AS_INITIATING_IO(griosp) \
			(griosp->flags &= ~STREAM_INITIATE_IN_PROGRESS)

#define STREAM_BEING_REMOVED(griosp)	\
			(griosp->flags & STREAM_REMOVE_IN_PROGRESS)

#define STREAM_INITIATE_IO(griosp)	\
			(griosp->flags & STREAM_INITIATE_IN_PROGRESS)


#define PER_FILE_SYS_STREAM(griosp)	\
			(griosp->flags & PER_FILE_SYS_GUAR)

#define PER_FILE_STREAM(griosp)		\
			(griosp->flags & PER_FILE_GUAR)


#define FIXED_ROTOR_STREAM( griosp )	\
			(griosp->flags & FIXED_ROTOR_GUAR)

#define SLIP_ROTOR_STREAM( griosp )	\
			(griosp->flags & SLIP_ROTOR_GUAR)

#define ROTOR_STREAM(griosp)		\
		(FIXED_ROTOR_STREAM( griosp) || SLIP_ROTOR_STREAM(griosp))

#define NON_ROTOR_STREAM(griosp)	 \
			(griosp->flags & NON_ROTOR_GUAR)



#define RT_SCHED_STREAM(griosp)	\
			(griosp->flags & REALTIME_SCHED_GUAR)

#define DL_SCHED_STREAM(griosp)	\
			(griosp->flags & DEADLINE_SCHED_GUAR)

#define NS_SCHED_STREAM(griosp)	\
			(griosp->flags & NON_SCHED_GUAR)


#define SLIPPED_ONCE_STREAM(griosp) \
			(griosp->flags & STREAM_SLIPPED_ONCE)

#define MARK_SLIPPED_ONCE_STREAM(griosp) \
			(griosp->flags  |= STREAM_SLIPPED_ONCE)

#define CLEAR_SLIPPED_ONCE_STREAM(griosp) \
			(griosp->flags &= ~STREAM_SLIPPED_ONCE)

/*
 * per disk stream information
 * the kernel allocates and maintaines one such structure for
 * each (stream id, disk used by that stream) pair.
 */
typedef struct grio_stream_disk_info {
	struct grio_stream_disk_info	*nextdiskinstream;
	grio_stream_info_t		*thisstream;
	struct grio_stream_disk_info	*nextstream;
	grio_disk_info_t		*griodp;

	lock_t	lock;				/* lock to protect structure            */
	time_t	period_end;			/* time current period will end         */
	time_t	iops_time;			/* length of period for this stream     */
	buf_t	*realbp;			/* orignal bp from xlv for this request */
	buf_t	*bp_list;			/* list of sub bps for this request     */
	buf_t	*queuedbps_front;		/* ptrs to queue of bps using this */ 
	buf_t	*queuedbps_back;		/* stream id                            */
	int	iops_remaining_this_period;	/* num grios left for this req this prd */
	time_t	time_priority_start;
	time_t	last_op_time;			/* time last out of band op */
	int	num_iops;			/* num grios for this req this period   */
	int	opt_io_size;			/* size of grio request                 */
} grio_stream_disk_info_t;


extern int 	grio_strat( buf_t *);
extern int 	grio_config( sysarg_t, sysarg_t, sysarg_t, sysarg_t, sysarg_t );
extern void 	grio_iodone( buf_t *);

/* This structure defines a linked list of rate guaranteed requests which
 * have been issued by clients but not yet satisfied by the daemon.
 */
typedef struct grio_cmd_queue {
        sema_t          	sema;
        grio_cmd_t      	griocmd;
        struct grio_cmd_queue 	*forw;
        struct grio_cmd_queue 	*back;
} grio_cmd_queue_t;

#define GRIO_NONGUARANTEED_STREAM(griosp) \
	( EQUAL_STREAM_ID( griosp->stream_id, non_guaranteed_id ) )

/*
 * Private data attached to the buffer structures.
 *	This structure may NOT contain any data unique specific to a 
 *	per disk I/O request. It is common to all the bps generated from
 *	a single user I/O request.
 */
typedef struct grio_buf_data {
	uuid_t	grio_id;		/* stream id of stream associated */
#ifdef GRIO_DEBUG			/* with this I/O request */
	time_t	start_time;
	time_t	enter_queue_time;
#endif
} grio_buf_data_t;



#ifdef GRIO_DEBUG
#define	INIT_GRIO_TIMESTAMP( bp  )  {			\
	if ( BUF_IS_GRIO( bp ) ) {			\
		BUF_GRIO_PRIVATE(bp)->start_time = lbolt;	\
	}						\
}
		
#define CHECK_GRIO_TIMESTAMP( bp, maxtime )	{			\
	if ( BUF_IS_GRIO( bp ) ) {					\
		if ((lbolt - BUF_GRIO_PRIVATE(bp)->start_time) > maxtime) {\
			printf("GRIO TIMESTAMP %d TICKS: file %s, line %d\n", \
				lbolt - BUF_GRIO_PRIVATE(bp)->start_time,\
				__FILE__,__LINE__);			\
		}							\
	}								\
}

#else
#define	INIT_GRIO_TIMESTAMP( bp )
#define	CHECK_GRIO_TIMESTAMP( bp, maxtime )
#endif


#define	BUF_GRIO_PRIVATE(bp) 	((grio_buf_data_t *)((bp)->b_grio_private))

/*
 *  Global lock and unlock macros
 */
#ifdef SIM

#define GRIO_GLOB_LOCK()        	1
#define GRIO_GLOB_UNLOCK( s )   	s
#define	GRIO_DISK_LOCK()		1
#define	GRIO_DISK_UNLOCK( s )		s
#define	GRIO_STREAM_LOCK()		1
#define	GRIO_STREAM_UNLOCK( s )		s
#define	GRIO_STREAM_DISK_LOCK()		1
#define	GRIO_STREAM_DISK_UNLOCK( s )	s
#define GRIO_TICKETT_LOCK() 		1
#define GRIO_TICKETT_UNLOCK(s)		s

#else

#define GRIO_GLOB_LOCK()		mutex_spinlock(&grio_global_lock)
#define GRIO_GLOB_UNLOCK(s)		mutex_spinunlock(&grio_global_lock, s)

#define GRIO_STABLE_LOCK()		mutex_spinlock(&grio_stream_table_lock)
#define GRIO_STABLE_UNLOCK(s) \
				mutex_spinunlock(&grio_stream_table_lock, s)

#define GRIO_DISK_LOCK(griodp)		mutex_spinlock(&griodp->lock)
#define GRIO_DISK_UNLOCK(griodp,s)	mutex_spinunlock(&griodp->lock, s)

#define GRIO_STREAM_LOCK(griosp)	mutex_spinlock(&griosp->lock)
#define GRIO_STREAM_UNLOCK(griosp,s)	mutex_spinunlock(&griosp->lock, s)

#define GRIO_STREAM_DISK_LOCK(griosdp)		mutex_spinlock(&griosdp->lock)
#define GRIO_STREAM_DISK_UNLOCK(griosdp,s)    mutex_spinunlock(&griosdp->lock,s)

#endif

/*
 * The INDEX MASK is equivalent to
 * (SCSI_UNIT_MASK | (DKSC_CTLR_MASK << (DKSC_CTLR_SHIFT - DKSC_UNIT_SHIFT)))
 */
#define INDEX_MASK      0x7ff

/*
 * Macro to convert the device number to an
 * index into the grio_info structure.
 */
#define GRIO_INDEX(gdev) ((gdev >> DKSC_UNIT_SHIFT) & INDEX_MASK)

/*
 * Macros to set or clear the bits in the b_flags2 field of the buf structure.
 */
#define BUF_IS_GRIO( bp )               (bp->b_flags2 &   B_GR_BUF)
#define BUF_IS_GRIO_QUEUED( bp )        (bp->b_flags2 &   B_GR_Q)
#define BUF_IS_GRIO_ISSUED( bp )        (bp->b_flags2 &   B_GR_ISD)
#define CLR_BUF_GRIO_QUEUED(bp)         (bp->b_flags2 &= ~B_GR_Q)
#define CLR_BUF_GRIO_ISSUED(bp)         (bp->b_flags2 &= ~B_GR_ISD)
#define MARK_BUF_GRIO_ISSUED(bp)        (bp->b_flags2 |=  B_GR_ISD)
#define MARK_BUF_GRIO_QUEUED(bp)        (bp->b_flags2 |=  B_GR_Q)


#ifdef DEBUG
#define dbg1printf(_x)		if (grio_dbg_level > 1 ) { printf _x ; }
#define dbg2printf(_x)		if (grio_dbg_level > 2 ) { printf _x ; }
#define dbg3printf(_x)		if (grio_dbg_level > 3 ) { printf _x ; }
#else
#define dbg1printf(_x)
#define dbg2printf(_x)
#define dbg3printf(_x)
#endif



#endif /* _KERNEL */

#define GRIO_MONITOR_COUNT	 100

typedef struct grio_monitor_times {
	time_t		starttime;
	time_t		endtime;
	__int64_t	size;
} grio_monitor_times_t;

typedef struct  grio_monitor {
	grio_monitor_times_t	times[GRIO_MONITOR_COUNT];
	int			start_index;
	int			end_index;
	stream_id_t		stream_id;
	int			monitoring;
} grio_monitor_t;

#endif /* _GRIO_H_ */
