#ifndef __SYS_TILE_H__
#define __SYS_TILE_H__

#ifdef __cplusplus
extern "C" {
#endif

/*
 * sys/tile.h
 *
 * 	provide 64k physically contiguous "tiles" of memory
 *
 * Copyright 1995, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED
 *
 * UNPUBLISHED -- Rights reserved under the copyright laws of the United
 * States.   Use of a copyright notice is precautionary only and does not
 * imply publication or disclosure.
 *
 * U.S. GOVERNMENT RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in FAR 52.227.19(c)(2) or subparagraph (c)(1)(ii) of the Rights
 * in Technical Data and Computer Software clause at DFARS 252.227-7013 and/or
 * in similar or successor clauses in the FAR, or the DOD or NASA FAR
 * Supplement.  Contractor/manufacturer is Silicon Graphics, Inc.,
 * 2011 N. Shoreline Blvd. Mountain View, CA 94039-7311.
 *
 * THE CONTENT OF THIS WORK CONTAINS CONFIDENTIAL AND PROPRIETARY
 * INFORMATION OF SILICON GRAPHICS, INC. ANY DUPLICATION, MODIFICATION,
 * DISTRIBUTION, OR DISCLOSURE IN ANY FORM, IN WHOLE, OR IN PART, IS STRICTLY
 * PROHIBITED WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SILICON
 * GRAPHICS, INC.
 */

#ident	"$Revision: 1.3 $"

#define TILE_SIZESHIFT	16			/* log2(TILE_SIZE) */
#define TILE_SIZE	(1 << TILE_SIZESHIFT)	/* 64k bytes per tile */
#define TILE_PAGES	(TILE_SIZE / NBPP)	/* pages per tile */
#define TILE_PAGEMASK	(TILE_PAGES - 1)	/* tile page bits of a pfn */
#define TILE_PAGESHIFT	(TILE_SIZESHIFT - PNUMSHFT) /* log2(TILE_PAGES) */


#if defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS)

typedef ushort_t	tile_index_t;

typedef struct tile_data_s {
    tile_index_t	td_next;	/* next     in list of tiles */
    tile_index_t	td_prev;	/* previous in list of tiles */
    unsigned		td_locked:16;	/* locked pages bitmap */
    unsigned		td_free:16;	/* not locked or allocated */
    unsigned		td_class:4;	/* tile class (memory bank) */
    unsigned		td_nlocked:6;	/* number of locked pages */
    unsigned		td_nfree:6;	/* number of free pages */
} tile_data_t;


#define tile_pfnbit(pfn)		(1 << ((pfn) & TILE_PAGEMASK))

#define TILE_SETLOCKED(td, pfn)		((td)->td_locked |=  tile_pfnbit(pfn))
#define TILE_CLRLOCKED(td, pfn)		((td)->td_locked &= ~tile_pfnbit(pfn))
#define TILE_ISLOCKED(td, pfn)		((td)->td_locked &   tile_pfnbit(pfn))

#define TILE_SETFREE(td, pfn)		((td)->td_free |=  tile_pfnbit(pfn))
#define TILE_CLRFREE(td, pfn)		((td)->td_free &= ~tile_pfnbit(pfn))
#define TILE_ISFREE(td, pfn)		((td)->td_free &   tile_pfnbit(pfn))

/* tile pointer to tile_data index, and back. */
#define tiletotdi(td)			((td) - tile_data)
#define tditotile(tdi)			(&tile_data[tdi])

/* tile_data index to page frame number, and back. */
#define tditopfn(tdi)			((tdi) << TILE_PAGESHIFT)
#define pfntotdi(pfn)			((pfn) >> TILE_PAGESHIFT)

/* tile pointer to page frame number, and back. */
#define tiletopfn(td)			tditopfn(tiletotdi(td))
#define pfntotile(pfn)			tditotile(pfntotdi(pfn))

/* tile pointer to pfdat, and back. */
#define tiletopfd(td)			pfntopfdat(tiletopfn(td))
#define pfdtotile(pfd)			pfntotile(pfdattopfn(pfd))

/* tile class of this tile. */
#define tiletoclass(tpfn)		(pfntotile(tpfn)->td_class)

typedef struct tileq_s {
    tile_index_t	tq_head;	/* first tile in queue */
    tile_index_t	tq_tail;	/* last  tile in queue */
    int			tq_freepgs;	/* free pages on this queue */
    int			tq_ntiles;	/* count of tiles in queue */
#if defined(TILE_DEBUG) || defined(DEBUG)
    int			tq_ntilesmax;	/* max length this queue has been */
#endif
} tileq_t;

#define TQ_UNLOCKED	(0)		/* queue index of unlocked tiles */
#define TQ_LOCKED	TILE_PAGES	/* queue index of locked tiles */
#define TQ_HOLE		(TILE_PAGES+1)	/* queue index of tiles in hole */
#define TQ_NQUEUES	(TQ_HOLE+1)	/* total number of queues */

#define tq_empty(qi)			(tileq[qi].tq_head == 0)
#define tq_ntiles(qi)			(tileq[qi].tq_ntiles)
#define tq_freepgs(qi)			(tileq[qi].tq_freepgs)
#define tiletoqi(td)			((td)->td_nlocked)
#define tq_enqh(td)			tq_enqhi(td, tiletoqi(td))
#define tq_enqt(td)			tq_enqti(td, tiletoqi(td))

/*
 * TILE_CLASS_MAX is the maximum number of memory banks
 * returned by addr_to_bank().
 */
#ifdef IP22
#define TILE_CLASS_MAX		4
#endif
#ifdef IP32
#define TILE_CLASS_MAX		16
#endif
#ifndef TILE_CLASS_MAX
/* default to single class/bank */
#define TILE_CLASS_MAX		1
#endif

/* classmask for "don't care" class for tile_alloc() */
#define TILE_ANYCLASS		(-1)	/* don't care which tile class */

/* flags options for tile_alloc() and tile_free() */
#define TILE_SLEEP		0x0	/* ok to sleep for tiles (default) */
#define TILE_NOSLEEP		0x1	/* don't sleep for tiles */
#define TILE_CLASS_SINGLE	0x2	/* all tiles from same class */
#define TILE_ZERO		0x4	/* zero tile pages */
#define TILE_ECCSTALE		0x8	/* stale ecc tiles */

#ifdef _KERNEL

extern tile_data_t *tile_data;		/* tile table */

void tile_init(pfn_t, pfn_t, int);
pfd_t *tile_pageselect(phead_t *, int, int);
void tile_allocinc(pfd_t *);
int tile_pagefree(pfd_t *);
void tile_pagelock(pfd_t *);

int tile_alloc(int, pfn_t *, int, int);
void tile_free(int, pfn_t *, int);
int tile_nclasses(void);
caddr_t tile_mapin(pfn_t, int);
void tile_mapout(caddr_t);

#endif /* _KERNEL */

#endif /* C || C++ */

#ifdef __cplusplus
}
#endif

#endif /* __SYS_TILE_H__ */
